<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class ApontamentoFactory
{

    use TQuery;
    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function lastEntry($nr)
    {

        $sql = "SELECT DTHRENTRADA
                  FROM FCM_REGISTROHORAS
                  WHERE NRREGISTROHORAS = (SELECT MAX(NRREGISTROHORAS)
                                             FROM FCM_REGISTROHORAS RH
                                             JOIN FCM_APONTAMENTO AP
                                               ON AP.NRAPONTAMENTO = RH.NRAPONTAMENTO
                                            WHERE AP.CDOPERADOR = " . $nr . "
                                            AND AP.IDFALTA = 'N')";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getTodayRegister($nr)
    {

        $sql = "SELECT COUNT(NRAPONTAMENTO) QTDAP
                    FROM FCM_APONTAMENTO
                WHERE TRUNC(FCM_APONTAMENTO.DTAPONTAMENTO) = TRUNC(SYSDATE)
                AND CDOPERADOR = " . $nr;

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getNRAP($nr)
    {

        $sql = "SELECT NRAPONTAMENTO
                    FROM FCM_APONTAMENTO
                    WHERE TRUNC(FCM_APONTAMENTO.DTAPONTAMENTO) = TRUNC(SYSDATE)
                    AND CDOPERADOR = " . $nr;

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }


    public function insertNewAppointment($parametros)
    {

        $sqlAlter = "ALTER SESSION SET NLS_DATE_FORMAT = 'DD/MM/YYYY HH24:MI:SS'";

        $this->connection->executeStatement($sqlAlter);

        $sql = "INSERT INTO FCM_APONTAMENTO (NRAPONTAMENTO, DTAPONTAMENTO, CDOPERADOR,
                  IDFALTA, IDAPURADO, IDATIVO, NRORG, DTINCLUSAO)
                  VALUES (:NRAPONTAMENTO, NVL(TO_CHAR(:DTAPONTAMENTO),SYSDATE), :CDOPERADOR, :IDFALTA, :IDAPURADO,
                   'S', :NRORG, SYSDATE)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertNewRegister($parametros)
    {

        $sqlAlter = "ALTER SESSION SET NLS_DATE_FORMAT = 'DD/MM/YYYY HH24:MI:SS'";

        $this->connection->executeStatement($sqlAlter);

        $sql = "INSERT INTO FCM_REGISTROHORAS (NRREGISTROHORAS, NRAPONTAMENTO, DTHRENTRADA, IDTIPORESGISTRO,
                    HRTEMPOREGISTRO, DSOBSREGISTRO, IDALMOCO, DTHRSAIDA, NRORG, IDATIVO, DTINCLUSAO, DTULTATU)
                VALUES (:NRREGISTROHORAS, :NRAPONTAMENTO, TO_CHAR(:DTHRENTRADA), :IDTIPORESGISTRO, 
                          NVL(:HRTEMPOREGISTRO, ROUND(((TO_DATE(:DTHRSAIDA) - TO_DATE(:DATEENTRY))*24)*60 ,0)),
                  :DSOBSREGISTRO, :IDALMOCO, TO_CHAR(:DTHRSAIDA), :NRORG, 'S', SYSDATE, SYSDATE)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function inserirAnexos($parametros)
    {

        $sql = "INSERT INTO FCM_ANEXOSAPONTAMENTO(NRANEXOSAPONTAMENTO, NRREGISTROHORAS, DSCAMINHOANEXO, NRORG, IDATIVO, 
                    DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO, DSTYPE, NRSIZE) 
                VALUES (:NRANEXOSAPONTAMENTO, :NRREGISTROHORAS, :DSCAMINHOANEXO, :NRORG, 'S', SYSDATE,
                        :CDOPERINCLUSAO, :NRORGINCLUSAO, :DSTYPE, :NRSIZE)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getUltimoRegistro($cdoperador)
    {

        $sql = " SELECT IDTIPORESGISTRO, NRAPONTAMENTO, RPAD(DTHRENTRADA,10) AS DTHRENTRADA
                  FROM FCM_REGISTROHORAS
                  WHERE NRREGISTROHORAS = (SELECT MAX(NRREGISTROHORAS)
                                             FROM FCM_REGISTROHORAS RH
                                             JOIN FCM_APONTAMENTO AP
                                               ON AP.NRAPONTAMENTO = RH.NRAPONTAMENTO
                                            WHERE AP.CDOPERADOR =" . $cdoperador . "
                                            AND AP.IDFALTA = 'N')";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function optionLunch($nr)
    {

        $sql = "SELECT COUNT(NRREGISTROHORAS) AS QTDALMOCO
                  FROM FCM_REGISTROHORAS
                 WHERE NRREGISTROHORAS = (SELECT MAX(NRREGISTROHORAS)
                                           FROM FCM_REGISTROHORAS RH
                                           JOIN FCM_APONTAMENTO AP
                                                             ON AP.NRAPONTAMENTO = RH.NRAPONTAMENTO
                                                          WHERE AP.CDOPERADOR = " . $nr . "
                                                          AND AP.IDFALTA = 'N'
                                                         AND RPAD(RH.DTHRSAIDA, 10) = TO_CHAR(SYSDATE, 'DD/MM/YYYY'))

                                 GROUP BY IDTIPORESGISTRO, IDALMOCO";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getApontamentos($cdoperador)
    {

        $sql = "SELECT RH.NRREGISTROHORAS, RH.DTHRENTRADA, RH.DTHRSAIDA, 
                DECODE(RH.IDALMOCO, 'S', '(Almoço)', ' ') AS IDALMOCO, RH.DTULTATU, 
                DECODE (RH.IDTIPORESGISTRO, 'E', RH.DTHRENTRADA,  RH.DTHRSAIDA) AS  DTREGISTRO,
                    CASE RH.IDTIPORESGISTRO WHEN 'E' THEN 'Entrada'
                        WHEN 'S' THEN 'Saída'
                        ELSE NULL END AS IDTIPORESGISTRO
                    FROM FCM_REGISTROHORAS RH
                    JOIN FCM_APONTAMENTO AP
                      ON AP.NRAPONTAMENTO = RH.NRAPONTAMENTO
                     AND AP.CDOPERADOR = " . $cdoperador . "
                   WHERE RPAD(RH.DTHRENTRADA,10) = TO_CHAR(SYSDATE, 'DD/MM/YYYY')
                      OR RPAD(RH.DTHRSAIDA, 10) = TO_CHAR(SYSDATE, 'DD/MM/YYYY')
                ORDER BY RH.DTULTATU";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
}
