<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class Fm00040000AuditoriaFactory
{

    use TQuery;
    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection       = $entityManager->getConnection();
        $this->entityManager    = $entityManager;
    }

    public function getAuditorias($listaOperadores, $filtros)
    {
        $sql = "SELECT FCM_AUDITORIA.NRAUDITORIA, FCM_AUDITORIA.NMAUDITORIA, FCM_AUDITORIA.NRLOCALCLIENTE, FCM_LOCALCLI.NMLOCALCLIENTE, FCM_MODELOAUDIT.NRMODELOAUDIT, 
                        FCM_MODELOAUDIT.NMMODELOAUDIT, FCM_AUDITORIA.NRSEQCONTRATO, FCT_CONTRATO.NRCONTRATOFCT, FCM_AUDITORIA.CDOPERADOR, OPERADOR.NMOPERADOR, 
                        TO_CHAR(FCM_AUDITORIA.DTHORAAUDIT, 'DD/MM') DTAGEND, FCM_LOCALCLI.CDCLIENTE, FCM_AUDITORIA.IDOBRIGAOBSERVACAO, FCM_AUDITORIA.IDTIPORECORRENCIA, 
                        FCM_TIPOLOCALCLI.NMTIPOLOCALCLI, CLIENTE.NMRAZSOCCLIE, CLIENTE.NMFANTCLIE, FCM_AUDITORIA.IDTIPOMARCACAO,
                        CASE FCM_AUDITORIA.IDSITUACAOAUDIT WHEN '1' THEN 'Pendente'
                            WHEN '2' THEN 'Em andamento'
                            WHEN '3' THEN 'Concluído'
                            ELSE NULL END AS IDSITUACAOAUDIT
                    FROM FCM_AUDITORIA
                    LEFT JOIN FCM_LOCALCLI
                    ON FCM_LOCALCLI.NRLOCALCLIENTE = FCM_AUDITORIA.NRLOCALCLIENTE
                    JOIN CLIENTE
                    ON CLIENTE.CDCLIENTE = FCM_LOCALCLI.CDCLIENTE
                    JOIN FCM_TIPOLOCALCLI
                    ON FCM_LOCALCLI.NRTIPOLOCALCLI = FCM_TIPOLOCALCLI.NRTIPOLOCALCLI
                    LEFT JOIN FCM_MODELOAUDIT
                    ON FCM_MODELOAUDIT.NRMODELOAUDIT = FCM_AUDITORIA.NRMODELOAUDIT
                    LEFT JOIN FCT_CONTRATO
                    ON FCT_CONTRATO.NRSEQCONTRATO = FCM_AUDITORIA.NRSEQCONTRATO
                    LEFT JOIN OPERADOR
                    ON OPERADOR.CDOPERADOR = FCM_AUDITORIA.CDOPERADOR
                    WHERE FCM_AUDITORIA.CDOPERADOR IN ({$listaOperadores}) $filtros ";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getCategoriasAuditoria($parametros)
    {
        $sql = "SELECT FCM_RELACMODCATEG.*, FCM_CATEGORIAAUDIT.NMCATEGORIAAUDIT 
                FROM FCM_RELACMODCATEG
                JOIN FCM_AUDITORIA
                ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_RELACMODCATEG.NRMODELOAUDIT
                JOIN FCM_CATEGORIAAUDIT
                ON FCM_CATEGORIAAUDIT.NRCATEGORIAAUDIT = FCM_RELACMODCATEG.NRCATEGORIAAUDIT
                WHERE FCM_AUDITORIA.NRAUDITORIA = :NRAUDITORIA";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getTopicosAuditoria($parametros)
    {
        $sql = "SELECT FCM_RELACCATTOPIC.*, FCM_TOPICOAUDIT.NMTOPICOAUDIT
                    FROM FCM_RELACCATTOPIC
                    JOIN FCM_AUDITORIA
                    ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_RELACCATTOPIC.NRMODELOAUDIT
                    JOIN FCM_TOPICOAUDIT
                    ON FCM_TOPICOAUDIT.NRTOPICOAUDIT = FCM_RELACCATTOPIC.NRTOPICOAUDIT
                    WHERE FCM_AUDITORIA.NRAUDITORIA = :NRAUDITORIA
                    AND FCM_RELACCATTOPIC.NRCATEGORIAAUDIT = :NRCATEGORIAAUDIT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getCriteriosAuditoria($parametros)
    {
        $sql = "SELECT FCM_RELACRESULTCRIT.NRRELACRESULTCRIT, FCM_RELACRESULTCRIT.NRRESULTADOAUDIT, FCM_RELACRESULTCRIT.NRCRITERIOAUDIT, 
                    FCM_RELACRESULTCRIT.IDREALIZADO, FCM_RELACRESULTCRIT.NRTOPICOAUDIT, FCM_CRITERIOAUDIT.NMCRITERIOAUDIT, FCM_RESULTADOAUDIT.NRCATEGORIAAUDIT
                        FROM FCM_RELACRESULTCRIT
                        JOIN FCM_RESULTADOAUDIT
                        ON FCM_RESULTADOAUDIT.NRRESULTADOAUDIT = FCM_RELACRESULTCRIT.NRRESULTADOAUDIT
                        JOIN FCM_CRITERIOAUDIT
                        ON FCM_CRITERIOAUDIT.NRCRITERIOAUDIT = FCM_RELACRESULTCRIT.NRCRITERIOAUDIT
                        JOIN FCM_AUDITORIA
                        ON FCM_RESULTADOAUDIT.NRAUDITORIA = FCM_AUDITORIA.NRAUDITORIA
                        WHERE FCM_RESULTADOAUDIT.NRCATEGORIAAUDIT = (SELECT DISTINCT NRCATEGORIAAUDIT FROM FCM_RESULTADOAUDIT
                                                                        WHERE NRTOPICOAUDIT = :NRTOPICOAUDIT)
                        AND FCM_RELACRESULTCRIT.NRTOPICOAUDIT = :NRTOPICOAUDIT
                        AND FCM_RESULTADOAUDIT.NRAUDITORIA = :NRAUDITORIA";

         return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function updateAuditoria($parametros)
    {
        $sql = "UPDATE FCM_AUDITORIA
                    SET IDSITUACAOAUDIT = :IDSITUACAOAUDIT,
                        DTULTATU = SYSDATE,
                        CDOPERULTATU = :CDOPERULTATU,
                        NRORGULTATU = :NRORGULTATU
                    WHERE NRAUDITORIA = :NRAUDITORIA";
        
        $this->connection->executeStatement($sql, $parametros);
    }

    public function updateCriteriosResultado($parametros, $criterios)
    {
        $sql = "UPDATE FCM_RELACRESULTCRIT
                SET IDREALIZADO = :IDREALIZADO,
                    DTULTATU = SYSDATE,
                    CDOPERULTATU = :CDOPERULTATU,
                    NRORGULTATU = :NRORGULTATU
                WHERE NRRELACRESULTCRIT IN ($criterios)";
        
        $this->connection->executeStatement($sql, $parametros);
    }

    public function anexarImagemAuditoria($parametros)
    {
        $sql = "INSERT INTO FCM_FOTORESULTAUDIT (NRFOTORESULTAUDIT, DSTYPE, NRSIZE, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO, NRCATEGORIAAUDIT, NRAUDITORIA, DSFOTOAUDITORIA)
        VALUES (:NRFOTORESULTAUDIT, :DSTYPE, :NRSIZE, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO, :NRCATEGORIAAUDIT, :NRAUDITORIA, :DSFOTOAUDITORIA)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getImagensAuditoria($parametros)
    {
        $sql = "SELECT * FROM FCM_FOTORESULTAUDIT
        WHERE NRAUDITORIA = :NRAUDITORIA";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function excluirImagemAuditoria($parametros)
    {
        $sql = "DELETE FROM  FCM_FOTORESULTAUDIT 
                    WHERE NRFOTORESULTAUDIT = :NRFOTORESULTAUDIT";
        
        $this->connection->executeStatement($sql, $parametros);
    }
}
