import Vue from 'vue';
import Router from 'vue-router';
import prodEnvironment from '@/config/prodEnvironment.json';
import devEnvironment from '@/config/devEnvironment.json';
import { DialogService, LoadingService } from '@zeedhi/common';
import {
    Config, Http, JsonCacheService, Loader,
} from '@zeedhi/core';
import { TekLibLocalStorage, TekLibAuth } from '@zeedhi/tek-lib';
import routes from './routes';
import Store from '../store';
import { Utilities } from '../util/Utilities';

Vue.use(Router);

const router = new Router({
    mode: 'history',
    base: Config.env.BASE_URL,
    routes,
});

const isOpeningNewModule = function isOpeningNewModule(to: any, from: any) {
    const isFromProduct = (to.name === 'microservice' && from.name !== 'microservice');
    // eslint-disable-next-line max-len
    const isFromAnotherModule = ((to.name === 'microservice' && from.name === 'microservice') && to.path.split('/')[2] !== from.path.split('/')[2]);
    const isNotLoginRoute = (to.fullPath !== Config.env.modules.login.routePath);
    return ((isFromProduct || isFromAnotherModule) && isNotLoginRoute);
};

JsonCacheService.saveJSONCache([
    { path: 'fm_00001_000_tarefas/fm_00001_000_tarefas' },
    { path: 'fm_00001_000_tarefas/fm_00001_000_detalhamento_tarefa' },
    { path: 'fm_00001_000_tarefas/fm_00001_000_reportar_tarefa' },
    { path: 'fm_00000_001_app_offline/fm_00000_001_app_offline_parametrizacao' },
]);

// eslint-disable-next-line max-len
const callLibStartSessionFromModule = async function callLibStartSessionFromModule(to: any) {
    const data = {
        USER: localStorage.getItem('LOGIN_USER'),
        LOGIN_LANGUAGE: localStorage.getItem('LOGIN_LAGUAGE'),
        LOGIN_TOKEN: localStorage.getItem('LOGIN_TOKEN'),
    };
    await Http.post(`${Config.env.modules[to.path.split('/')[2]].endPoint}/lib_startSession`, data);
};

router.beforeEach(async (to, from, next) => {
    if (/Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent) && to.name === 'microservice') {
        Utilities.mensagens('Error!', 'A tela desejada ainda nÃ£o foi preparada para o ambiente mobile. <br> <br> Favor acessar essa tela por um notebook ou desktop.');
        return;
    }

    if (isOpeningNewModule(to, from) && TekLibLocalStorage.getItem('IS_NEXT')) {
        await callLibStartSessionFromModule(to);
    }

    /** open old application - init */
    if (window.parent !== window && to.name !== 'token-error' && to.name !== 'microservice') { // if application is called inside a iframe
        if (to.query.token) {
            // if token was passed
            localStorage.setItem('LOGIN_TOKEN', to.query.token.toString());
            localStorage.setItem('LOGIN_USER', to.query.user.toString());
            localStorage.setItem('LOGIN_LANGUAGE', to.query.languange.toString());
            localStorage.setItem('PRODUCT_ID', to.query.productId.toString());
            if (to.query.hash !== '0') {
                localStorage.setItem('LOGIN_HASH', to.query.hash.toString());
            }
            Store.commit('setAppLoginToken', to.query.token);
            Store.commit('setAppDisplayOptions', { menu: to.query.menu, header: to.query.header });
        }

        if (!Store.state.appLoginToken) {
            // if there is no token... redirect to another page
            Store.commit('setAppDisplayOptions', { menu: '0', header: '0' });
            next('/token-error');
            return;
        }

        // remove parameters from requested page
        delete to.query.menu;
        delete to.query.header;
        delete to.query.token;

        TekLibLocalStorage.setItem('IS_NEXT', false);
        TekLibLocalStorage.setItem('LOGIN_KEEP_CONNECTED', 'Yes');
        TekLibLocalStorage.removeItem('routeParams');

        next();
    } else {
        TekLibLocalStorage.setItem('IS_NEXT', true);
    }
    /** open old application - end */

    TekLibAuth.setInitialData(to.path, Config.env.loginRoutePath);

    if (to.path !== Config.env.loginRoutePath && !localStorage.getItem('LOGIN_TOKEN')) {
        next(Config.env.loginRoutePath);
    } else if (to.path === Config.env.loginRoutePath && localStorage.getItem('LOGIN_TOKEN')) {
        next(Config.env.baseUrl);
    } else {
        next();
    }

    // Responsável por controlar a troca de rota entre o Mirage e o Servidor.
    // window.localStorage.setItem('MIRAGEJS', 'N');
    if (window.localStorage.getItem('MIRAGEJS') !== null) {
        Http.get('/testConnection', { timeout: 5000 }).then(() => {
            if (window.localStorage.getItem('MIRAGEJS') === 'S') {
                window.localStorage.setItem('MIRAGEJS', 'N');
                const env = process.env.NODE_ENV === 'production' ? prodEnvironment : devEnvironment;
                Config.endPoint = env.endPoint;
                Http.setBaseURL(Config.endPoint);
                DialogService.show({
                    name: 'warning-dialog',
                    type: 'warning',
                    title: 'Aviso!',
                    text: 'Podem haver dados para serem sincronizados. Deseja sincronizar agora?',
                    buttons: [
                        {
                            name: 'no',
                            component: 'ZdButton',
                            label: 'Não',
                            events: {
                                click: () => { DialogService.hide(); },
                            },
                        },
                        {
                            name: 'yes',
                            component: 'ZdButton',
                            label: 'Sim',
                            events: {
                                click: () => {
                                    const param = Loader.getInstance('ParametrizacaoController');
                                    DialogService.hide();
                                    param.baixarDadosIndexDB();
                                },
                            },
                            outline: true,
                        },
                    ],
                });
            }
        }).catch(() => {
            LoadingService.hide();
            if (window.localStorage.getItem('MIRAGEJS') !== 'S') {
                Utilities.mensagens('Offline', 'No momento você está sem conexão com o servidor. Porém, pode acessar as seguintes telas: <b> Minhas Tarefas </b> e trabalhar Offline.');
                window.localStorage.setItem('MIRAGEJS', 'S');
                Config.endPoint = 'http://mirage';
                Http.setBaseURL(Config.endPoint);
            }
        });
    }
});

router.afterEach((to, from) => {
    const htmlElement = document.getElementsByTagName('html')[0];
    if (to.name === 'microservice') {
        htmlElement.setAttribute('microservice', '');
        Store.commit('setRouterParams', to.params);
        const frame = (document.getElementById('microservice-frame') as HTMLIFrameElement);
        if (frame && frame.contentWindow && to.path.split('/')[2] === from.path.split('/')[2]) {
            frame.contentWindow.postMessage({
                type: 'microservice-route-change',
                routerParams: to.params,
            }, `${Config.env.modules[to.path.split('/')[2]].baseUrl}`);
        }
        window.document.title = `${Config.env.product.name} - ${to.params.page}`;
    } else {
        htmlElement.removeAttribute('microservice');
        window.document.title = Config.env.product.name;
    }

    if (to.name === 'login') {
        htmlElement.setAttribute('login', '');
    } else {
        htmlElement.removeAttribute('login');
    }
});

export const getRouteParams = () => {
    const { currentRoute } = router;
    const { params } = currentRoute;
    return params;
};

export const getRoutePath = () => {
    const { currentRoute } = router;
    const { path } = currentRoute;
    return path;
};

export default router;
